﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Security.Cryptography;
using TMPro;
using UnityEngine;
using UnityEngine.Networking;
using UnityEngine.UI;

public class CommonDocument : MonoBehaviour
{
    public MeshRenderer renderQuad;
    public TextMeshPro pagesText;
    public TextMeshPro titleText;
    public Texture ErrorTex;
    private List<string> pageUrls;
    private int currentPage = 0;

    private Material pageMaterial;
    private Texture currentTex;
    private Coroutine fetch;
    private String documentName = "unnamed document";

    public float maxYScale = 0.45f;
    public float maxXScale = 0.45f;


    void Awake()
    {
        renderQuad.material = new Material(renderQuad.material);
        pageMaterial = renderQuad.material;
        pageUrls = new List<string>();
        maxYScale = renderQuad.transform.localScale.y * 0.95f;
        maxXScale = renderQuad.transform.localScale.x * 0.95f;
    }

    Vector2 FitHorizontal(float h, float w)
    {
        var ratio = h / w;
        var x = maxXScale;
        var y = maxXScale * ratio;
        return new Vector2(x, y);
    }

    Vector2 FitVertical(float h, float w)
    {
        var ratio = w / h;
        var x = maxYScale * ratio;
        var y = maxYScale;
        return new Vector2(x, y);
    }


    void Rescale()
    {
        currentTex = pageMaterial.mainTexture;
        var PixelScale = 1f;
        var h = currentTex.height * 1.0f;
        var w = currentTex.width * 1.0f;
        var ratio = FitVertical(h, w);
        if (ratio.x > maxXScale)
            ratio = FitHorizontal(h, w);

        renderQuad.transform.localScale = PixelScale * new Vector3(
            ratio.x,
            ratio.y,
            1 / PixelScale);
    }

    void Update()
    {
        //Rescale();
    }

    public void Close()
    {
        gameObject.SetActive(false);
    }

    public void SetName(string newName)
    {
        documentName = newName;
        if (pageUrls.Count > 0)
            Rerender();
    }

    public void SetPages(List<string> urls)
    {
        pageUrls = urls;
        Rerender();
    }

    public void AddPage(string url)
    {
        pageUrls.Add(url);
        Rerender();
    }


    public void NextPage()
    {
        if (currentPage == pageUrls.Count - 1)
            currentPage = 0;
        else
            currentPage += 1;
        Rerender();
    }

    public void PrevPage()
    {
        if (currentPage == 0)
            currentPage = pageUrls.Count - 1;
        else
            currentPage -= 1;
        Rerender();
    }

    public void Rerender()
    {
        if (fetch != null)
        {
            StopCoroutine(fetch);
        }

        StartCoroutine(RerenderRoutine());
    }

    private IEnumerator RerenderRoutine()
    {
        var url = pageUrls[currentPage];
        UnityWebRequest request = UnityWebRequestTexture.GetTexture(url);
        yield return request.SendWebRequest();
        Texture newTex;
        if (request.isNetworkError || request.isHttpError)
        {
            Debug.Log(request.error);
            newTex = ErrorTex;
        }
        else
        {
            newTex = ((DownloadHandlerTexture) request.downloadHandler).texture;
        }
        pageMaterial.mainTextureScale = new Vector2(1, 1);
        pageMaterial.mainTexture = newTex;

        var newPagesText = $"{currentPage + 1}/{pageUrls.Count}";

        Rescale();
        pagesText.text = newPagesText;
        titleText.text = documentName;
    }
}