﻿using System.Collections;
using System.Collections.Generic;
using Microsoft.MixedReality.Toolkit;
using Microsoft.MixedReality.Toolkit.Input;
using Microsoft.MixedReality.Toolkit.Utilities;
using Microsoft.MixedReality.Toolkit.Utilities.Solvers;
using UnityEngine;
using UnityEngine.Networking;

public class DocumentManager : MonoBehaviour, IMixedRealitySpeechHandler
{
    public Transform DocumentOrigin;
    public GameObject DocumentPanePrefab;
    private GridObjectCollection gridObjectCollection;
    private Transform camTransform;
    private Vector3 defaultPaneScale;

    void Start()
    {
        CoreServices.InputSystem?.RegisterHandler<IMixedRealitySpeechHandler>(this);
        camTransform = Camera.main.transform;
        gridObjectCollection = DocumentOrigin.GetComponent<GridObjectCollection>();
        defaultPaneScale = DocumentPanePrefab.transform.localScale;
        Mock();
    }

    private void Unfollow(GameObject go)
    {
        var toggle = go.GetComponent<RadialView>();
        toggle.enabled = false;
    }
    
    private void Unfollow(Transform go)
    {
        var toggle = go.GetComponent<RadialView>();
        toggle.enabled = false;
    }
    
    public void Organize()
    {
        foreach(Transform child in DocumentOrigin)
        {
            child.gameObject.SetActive(true);
            child.localScale = defaultPaneScale;
            try
            {
                Unfollow(child);
            }
            catch
            {
                foreach (Transform child2 in child)
                {
                    Unfollow(child2);
                    break;
                }
            }
        }
        DocumentOrigin.position = camTransform.position;
        DocumentOrigin.rotation = camTransform.rotation;
        gridObjectCollection.UpdateCollection();
    }

    private CommonDocument Spawn()
    {
        var createdDoc = Instantiate(DocumentPanePrefab,
            camTransform.position + camTransform.forward * 2f,
            camTransform.rotation,
            DocumentOrigin);
        var doc = createdDoc.GetComponent<CommonDocument>();
        return doc;
    }
    
    public CommonDocument CreateDocument(string docName)
    {
        var doc = Spawn();
        doc.SetName(docName);
        return doc;
    }

    public CommonDocument CreateDocument(string docName, List<string> urls)
    {
        var doc = CreateDocument(docName);
        doc.SetPages(urls);
        return doc;
    }
    
    void Mock()
    {
        CreateDocument("Отчёт.pdf", 
            new List<string>
            {
                "https://i.pinimg.com/originals/a3/1e/47/a31e470fc1911b2d1c4e683155216716.png",
                "https://ak.picdn.net/shutterstock/videos/1037009135/thumb/6.jpg",
                "https://pipcat.ru//wp-content/uploads/2016/08/DSC_77911.jpg",
                "https://icatcare.org/app/uploads/2018/07/Thinking-of-getting-a-cat.png",
                "https://i.pinimg.com/736x/33/08/8a/33088a684f8af83e6b6be61828190a7b.jpg"
            });
        CreateDocument("Руководство по эксплуатации железных машин", 
            new List<string>
            {
                "https://images.pexels.com/photos/1459505/pexels-photo-1459505.jpeg?auto=compress&cs=tinysrgb&dpr=1&w=500",
                "https://i.pinimg.com/236x/f0/8b/33/f08b339a1cfe789e5e5d725fee58f53d--futuristic-architecture-amazing-architecture.jpg",
                "https://image.shutterstock.com/image-photo/wide-angle-panorama-autumn-forestmisty-260nw-1195159864.jpg",
                "https://unity3d.com/profiles/unity3d/themes/unity/images/unity/beta/alpha-beta/2020.1b-improvements.jpg",
                "https://cdn.images.fecom-media.com/FE00015871/images/HE1815179_1426638-GLS-STA-P01.jpg"
            });
        CreateDocument("Мемас", 
            new List<string>
            {
                "https://upload.wikimedia.org/wikipedia/ru/thumb/4/4d/Wojak.png/200px-Wojak.png"
            });
        CreateDocument("Инструкция (для внутреннего использования)", 
            new List<string>
            {
                "https://2x2tv.ru/upload/setka-editor/daa/daaa093de2712df250954eceb82d01d4.jpg",
                "https://randomwordgenerator.com/img/picture-generator/57e4d04a4950ae14f1dc8460962e33791c3ad6e04e5074417d2d73d3934ac3_640.jpg"
            });
        CreateDocument("Протокол собрания.pdf", 
            new List<string>
            {
                "https://randomwordgenerator.com/img/picture-generator/53e1dc454c5aa414f1dc8460962e33791c3ad6e04e507440752b7edc9749c7_640.jpg",
                "https://randomwordgenerator.com/img/picture-generator/55e6d5414b52a414f1dc8460962e33791c3ad6e04e507440752f78d0944ec4_640.jpg",
                "https://randomwordgenerator.com/img/picture-generator/54e6d4474f50a814f1dc8460962e33791c3ad6e04e5074417c2c7fd59e48c3_640.jpg",
                "https://randomwordgenerator.com/img/picture-generator/57e5d74a4f57af14f1dc8460962e33791c3ad6e04e50744172297cd69745c5_640.jpg"
            });
        CreateDocument("Сложные картинки есть не только тут", 
            new List<string>
            {
                "https://upload.wikimedia.org/wikipedia/commons/thumb/d/d2/Reddot-small.svg/1200px-Reddot-small.svg.png",
                "https://upload.wikimedia.org/wikipedia/commons/d/d5/Japan_small_icon.png",
                "https://i.imgur.com/b96dOB0.gif"
            });
        CreateDocument("Тест проблемы с картинкой и длинным названием", 
            new List<string>
            {
                "hello"
            });
        
    }

    private IEnumerator FetchDocuments()
    {
        var url = "";
        while (gameObject.activeSelf)
        { 
            UnityWebRequest request = UnityWebRequest.Get(url);
            yield return request.SendWebRequest();
            if (request.isNetworkError || request.isHttpError)
            {
                Debug.Log(request.error);
                continue;
            }

            var docs = request.downloadHandler.text;
            
            //docs to hashmap
            //find difference
            //foreach in difference request urls
            //  and create pane
            
            yield return new WaitForSeconds(6f);
        }
    }
    

    public void OnSpeechKeywordRecognized(SpeechEventData eventData)
    {
        var command = eventData.Command.Keyword.ToLower();
        if (command.StartsWith("document") || command.StartsWith("organize"))
        {
            Organize();
        }
    }
}