﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.MixedReality.WebRTC;
using Microsoft.MixedReality.WebRTC.Unity;
using NativeWebSocket;
using UnityEngine;
using UnityEngine.Networking;

public class NonSignaler : Signaler
{
    public string wssAddress = "wss://nonpropria.space/signal";

    public PointerManager PointerManager;
    
    WebSocket websocket;

    private static string oldOffer = "";
    private static string oldAnswer = "";


    private async void Start()
    {
        websocket = new WebSocket(wssAddress);

        websocket.OnOpen += () =>
        {
            Debug.Log("Connection open!");
        };

        websocket.OnError += (e) => { Debug.Log("Error! " + e); };

        websocket.OnClose += (e) => { Debug.Log("Connection closed! (" + e + ")"); };

        websocket.OnMessage += this.InnerOnMessage;

        await websocket.Connect();
    }


    public void ConfigurePeer()
    {
        // var micSource = localAudioObj.GetComponent<MicrophoneSource>();
        // var audioConfig = new LocalAudioTrackInitConfig
        // {
        //     trackName = "microphone_track"
        // };
        //
        // localAudioTrack = LocalAudioTrack.CreateFromSource(micSource.Source, audioConfig);
        //
        // TransceiverInitSettings option2 = new TransceiverInitSettings();
        // option2.Name = "microphone_track";
        // option2.StreamIDs = new List<string> {"webcam_name2"};
        //
        // var audioTransceiver = PeerConnection.Peer.AddTransceiver(MediaKind.Audio, option2);
        // audioTransceiver.DesiredDirection = Transceiver.Direction.SendOnly;
        // audioTransceiver.LocalAudioTrack = localAudioTrack;

        // var vidSource = localVideoObj.GetComponent<WebcamSource>();
        // var videoConfig = new LocalVideoTrackInitConfig
        // {
        //     trackName = "webcam_track"
        // };
        //
        // localVideoTrack = LocalVideoTrack.CreateFromSource(vidSource.Source, videoConfig);
        //
        // TransceiverInitSettings option1 = new TransceiverInitSettings();
        // option1.Name = "webcam_track";
        // option1.StreamIDs = new List<string> {"webcam_name"};
        // var videoTransceiver = PeerConnection.Peer.AddTransceiver(MediaKind.Video, option1);
        // videoTransceiver.DesiredDirection = Transceiver.Direction.SendOnly;
        // videoTransceiver.LocalVideoTrack = localVideoTrack;

        print(PeerConnection.Peer.Initialized);
        print("Trans set.");


        // PeerConnection.StartConnection();
        // StartCoroutine(DelayedCall());
    }

    private IEnumerator DelayedCall()
    {
    // var task = Task.Run(() => );
    // yield return task;
    print("Socket connected");
    yield return new WaitForSeconds(3);
    PeerConnection.StartConnection();
    }


    private class NonMessage
    {
        /// <summary>
        /// Separator for ICE messages.
        /// </summary>
        public const string IceSeparatorChar = "|";

        /// <summary>
        /// The message type.
        /// </summary>
        public string type = Type.unknown.ToString();

        public string sdp;

        public int label;
        public string id;
        public string candidate;
        public float x;
        public float y;

        /// <summary>
        /// Possible message types as-serialized on the wire to <c>node-dss</c>.
        /// </summary>
        public enum Type
        {
            /// <summary>
            /// An unrecognized message.
            /// </summary>
            unknown = 0,

            /// <summary>
            /// A SDP offer message.
            /// </summary>
            offer,

            /// <summary>
            /// A SDP answer message.
            /// </summary>
            answer,

            /// <summary>
            /// A trickle-ice or ice message.
            /// </summary>
            candidate,
            
            pointer
        }

        /// <summary>
        /// Convert a message type from <see xref="string"/> to <see cref="Type"/>.
        /// </summary>
        /// <param name="stringType">The message type as <see xref="string"/>.</param>
        /// <returns>The message type as a <see cref="Type"/> object.</returns>
        public static Type MessageTypeFromString(string stringType)
        {
            if (string.Equals(stringType, "offer", StringComparison.OrdinalIgnoreCase))
            {
                return Type.offer;
            }
            else if (string.Equals(stringType, "answer", StringComparison.OrdinalIgnoreCase))
            {
                return Type.answer;
            }

            throw new ArgumentException($"Unknown signaler message type '{stringType}'", "stringType");
        }

        public static string MessageTypeFromSdpMessageType(SdpMessageType type)
        {
            switch (type)
            {
                case SdpMessageType.Offer: return "offer";
                case SdpMessageType.Answer: return "answer";
                default: return "unknown";
            }
        }

        public IceCandidate ToIceCandidate()
        {
            if (type != Type.candidate.ToString())
            {
                throw new InvalidOperationException("The node-dss message it not an ICE candidate message.");
            }

            return new IceCandidate
            {
                SdpMid = this.id,
                SdpMlineIndex = this.label,
                Content = this.candidate
            };

            //var parts = Data.Split(new string[] {IceSeparatorChar}, StringSplitOptions.RemoveEmptyEntries);
            // Note the inverted arguments; candidate is last in IceCandidate, but first in the node-dss wire message
            // return new IceCandidate
            // {
            //     SdpMid = parts[2],
            //     SdpMlineIndex = int.Parse(parts[1]),
            //     Content = parts[0]
            // };
        }

        public NonMessage(SdpMessage message)
        {
            type = MessageTypeFromSdpMessageType(message.Type);
            if (type == "offer")
            {
                oldOffer = message.Content;
            }
            else if (type == "answer")
            {
                oldAnswer = message.Content;
            }

            sdp = message.Content;
        }

        public NonMessage(IceCandidate candidate)
        {
            type = Type.candidate.ToString();
            label = candidate.SdpMlineIndex;
            id = candidate.SdpMid;
            this.candidate = candidate.Content;
        }
    }

    public override Task SendMessageAsync(SdpMessage message)
    {
        return SendMessageImplAsync(new NonMessage(message));
    }

    public override Task SendMessageAsync(IceCandidate candidate)
    {
        return SendMessageImplAsync(new NonMessage(candidate));
    }

    private Task SendMessageImplAsync(NonMessage message)
    {
        // This method needs to return a Task object which gets completed once the signaler message
        // has been sent. Because the implementation uses a Unity coroutine, use a reset event to
        // signal the task to complete from the coroutine after the message is sent.
        // Note that the coroutine is a Unity object so needs to be started from the main Unity app thread.
        // Also note that TaskCompletionSource<bool> is used as a no-result variant; there is no meaning
        // to the bool value.
        // https://stackoverflow.com/questions/11969208/non-generic-taskcompletionsource-or-alternative
        // var tcs = new TaskCompletionSource<bool>();
        // _mainThreadWorkQueue.Enqueue(() => StartCoroutine(PostToServerAndWait(message, tcs)));
        // return tcs.Task;

        //
        //
        //
        //
        var jsonData = JsonUtility.ToJson(message);
        //var data = System.Text.Encoding.UTF8.GetBytes(jsonData);
        print("Send: " + jsonData);
        return websocket.SendText(jsonData);
    }

    /// <summary>
    /// Internal coroutine helper for receiving HTTP data from the DSS server using GET
    /// and processing it as needed
    /// </summary>
    /// <returns>the message</returns>
    private void CO_GetAndProcessFromServer(string nonMessageText)
    {
        var msg = JsonUtility.FromJson<NonMessage>(nonMessageText);

        // if the message is good
        if (msg != null)
        {
            // depending on what type of message we get, we'll handle it differently
            // this is the "glue" that allows two peers to establish a connection.
            DebugLogLong($"Received SDP message: type={msg.type}" +
                         $"#{msg.label} #{msg.id} #{msg.candidate} #{msg.sdp}");

            if (msg.type == NonMessage.Type.offer.ToString())
            {
                // Apply the offer coming from the remote peer to the local peer
                // var customSdp = msg.sdp.Replace("\r\n", "\n");
                var sdpOffer = new SdpMessage {Type = SdpMessageType.Offer, Content = msg.sdp};
                if (oldOffer == sdpOffer.Content)
                {
                    print("Ignoring call to ourselves");
                    return;
                }
                print("Trying to answer offer");
                PeerConnection.HandleConnectionMessageAsync(sdpOffer).ContinueWith(_ =>
                    {
                        print("Calling CreateAnswer");
                        // If the remote description was successfully applied then immediately send
                        // back an answer to the remote peer to acccept the offer.
                        _nativePeer.CreateAnswer();
                    },
                    TaskContinuationOptions.OnlyOnRanToCompletion |
                    TaskContinuationOptions.RunContinuationsAsynchronously);
            }
            else if (msg.type == NonMessage.Type.answer.ToString())
            {
                // No need to wait for completion; there is nothing interesting to do after it.
                var sdpAnswer = new SdpMessage {Type = SdpMessageType.Answer, Content = msg.sdp};
                if (oldAnswer == sdpAnswer.Content)
                {
                    print("Ignoring answer to ourselves");
                    return;
                }

                _ = PeerConnection.HandleConnectionMessageAsync(sdpAnswer);
            }
            else if (msg.type == NonMessage.Type.candidate.ToString())
            {
                // this "parts" protocol is defined above, in OnIceCandidateReadyToSend listener
                _nativePeer.AddIceCandidate(msg.ToIceCandidate());
            }
            else if (msg.type == NonMessage.Type.pointer.ToString())
            {
                var screenPos = new Vector2(msg.x, msg.y);
                PointerManager.UpdateRay(screenPos);
            }
            else
            {
                Debug.Log("Unknown message: " + msg.type + ": " + msg.sdp);
            }
        }
        else
        {
            Debug.LogError($"Failed to deserialize JSON message : {nonMessageText}");
        }
    }

    void InnerOnMessage(byte[] bytes)
    {
        // Reading a plain text message
        var message = System.Text.Encoding.UTF8.GetString(bytes);
        //Debug.Log("Received OnMessage! (" + bytes.Length + " bytes) " + message);
        this.CO_GetAndProcessFromServer(message);
    }


    protected override void Update()
    {
        base.Update();
#if !UNITY_WEBGL || UNITY_EDITOR
        websocket.DispatchMessageQueue();
#endif
    }

    private async void OnApplicationQuit()
    {
        await websocket.Close();
    }

    private void DebugLogLong(string str)
    {
#if !UNITY_EDITOR && UNITY_ANDROID
            // On Android, logcat truncates to ~1000 characters, so split manually instead.
            const int maxLineSize = 1000;
            int totalLength = str.Length;
            int numLines = (totalLength + maxLineSize - 1) / maxLineSize;
            for (int i = 0; i < numLines; ++i)
            {
                int start = i * maxLineSize;
                int length = Math.Min(start + maxLineSize, totalLength) - start;
                Debug.Log(str.Substring(start, length));
            }
#else
        Debug.Log(str);
#endif
    }
}